******************************************************************************************
* PROCEDURE MOVE ITEM v.1.2 May 10, 1995
*
* This version of MOVE_ITEM is public domain.  You may copy and distribute
* it freely (but if you like it we would appreciate a pat on the back as well.)
*
* Original 
*  Author: 	Claudio Sousa
* Company: 	Evolve Computer Solutions
*			678 Waverley Street
*			Winnipeg, Manitoba  R3M 3L6
*			Compuserve: 71673,2131
*			Internet: claudio@coned.uwinnipeg.ca
*
* Version 1.2 Enhancements
* Author: 	Kalen S. Krueger
* Company: 	Krueger Consulting, Inc.
*			3702 South Fife Street, Ste. 203
*			Tacoma, WA 98403
*			Compuserve: 70233,1060
*			Internet: kruegerks@aol.com
*
*
* Copyright: C.Sousa, Evolve Computer Solutions, April 1995
*            Kalen S. Krueger, Krueger Consulting, May 1995
*
*
* DESCRIPTION: This procedure is designed for use in a dual picklist screen.
* The design of these procedures allow multiple dualing-picklists to exist
* on the same screen and use the same procedures.
*
* This procedure uses extensive macro substitution to accomplish this task and is
* therefore not the most efficient, however, given that this procedure is designed
* for use within screen interfaces, speed may not be an issue in most cases.  This program
* also works for two dimensional arrays providing that both arrays have the same number of
* columns.
*-----------------------------------------------------------------------------------------
* WHAT'S NEW?  This procedure makes minor modifications to the previous implementation that
* should result in slightly better performance.  It also fixed a bug when dealing with multi-
* dimensional arrays, and made the two size variables PRIVATE eliminating the need for PUBLIC
* ones.  It has been tested with 1X1 and 1X2 arrays of up to 400 items and should work with 
* other dimensions as well.
*-----------------------------------------------------------------------------------------
* WARNING! WARNING! WARNING! WARNING! WARNING! WARNING! WARNING! WARNING! WARNING! WARNING!
* Calls to this function are not backward compatible with v.1.1.  If you are using
* this tool for the first time, or in a new application, ignore this section.  Otherwise, 
* existing picklists calling this function will need to be modified, and this can be done by:
*
*	1)	changing the VALID code snippets for each of the three objects in your picklist,
*		eliminating the passing of the size parameters.  Other minor modifications to 
*		these snippets should be made as well, and they are demonstrated in the sample.
*		The simplest way to update your picklists would be to cut-and-paste the VALID 
*		functions from each picklist object in the sample into your existing picklists'
*		objects, and then update them to reflect your variables' names.
*
*	2)  changing any calls to the size variables for either of the lists as they are no 
*		longer PUBLIC.  Replace these calls with  >>  = ALEN("Your Array Name", 1)  <<
*		The previous size variables can be eliminated.  These changes are also reflected 
*		in the sample.
*
*	3)  clearing the "# OF ELEMENTS" Clause for each of the lists in your picklists.
*-----------------------------------------------------------------------------------------
*
* Syntax:  DO MOVE_ITEM with expc1, expi1, expc2, expi2, [expL]
*			expc1 = name of source array in quotes
*			expi1 = numeric row index value of the item to be moved
*			expc2 = name of destination array in quotes
*			expi2 = numeric row index value destination array
*			expL  = logical expression True (default) will sort array after moving item
*									   False will NOT sort array after moving item
*
* Example:
*		CLEAR
*		CLEAR MEMORY
*		DIMENSION xLeft[3,2]
*		xLeft[1,1] = "First"
*		xLeft[1,2] = "next to first"
*		xLeft[2,1] = "Second"
*		xLeft[2,2] = "next to second"
*		xLeft[3,1] = "Third"
*		xLeft[3,2] = "next to third"
*		leftsize = 3
*		leftidx = 1
*
*		DIMENSION xRight[1,2]
*		xRight[1,1] = ""
*		xRight[1,2] = ""
*		rightsize = 1
*		rightidx = 1
*
*		?"********** Before Move Item ******************"
*		DISPLAY MEMORY LIKE x*
*		DO move_item WITH 'xLeft',leftidx, 'xRight', rightidx, rightsize
*		?"********** After Move Item *******************"
*		DISPLAY MEMORY LIKE x*
*		DO move_all WITH 'xRight', rightidx, 'xLeft',leftidx ,leftsize
*		?"********** After Move all *******************"
*		DISPLAY MEMORY LIKE x*
*
******************************************************************************************
PARAMETERS cSourceArray, nSourceIdx, cDestArray, nDestIdx, lSort
PRIVATE nColumns, nSourceRows, nDestRows

&& Exit if source array is empty
IF EMPTY(&cSourceArray)
	RETURN
ENDIF

&& Set sort flag
IF PARAMETERS() != 5
	lSort = .T.
ENDIF

&& Assign variables
nColumns    = ALEN(&cSourceArray, 2)
nSourceRows = ALEN(&cSourceArray, 1)
nDestRows   = ALEN(&cDestArray, 1)

&& Add row to destination array
IF NOT EMPTY(&cDestArray[1])
	nDestRows = nDestRows + 1
 	IF nColumns > 1 
		DIMENSION &cDestArray[nDestRows, nColumns]
	ELSE
		DIMENSION &cDestArray[nDestRows]
	ENDIF
ENDIF

&& Copy values from source array to destination array, sort if called for, and reset index
IF nColumns > 1
	= ACOPY(&cSourceArray, &cDestArray, AELEMENT(&cSourceArray, nSourceIdx, 1), nColumns, AELEMENT(&cDestArray, nDestRows, 1))
	IF lSort
		= ASORT(&cDestArray)
	ENDIF
	nDestIdx = ASUBSCRIPT(&cDestArray, ASCAN(&cDestArray, &cSourceArray[nSourceIdx, 1]), 1 )
ELSE
	= ACOPY(&cSourceArray, &cDestArray, nSourceIdx, 1, nDestRows)
	IF lSort
		= ASORT(&cDestArray)
	ENDIF
	nDestIdx = ASUBSCRIPT(&cDestArray, ASCAN(&cDestArray, &cSourceArray[nSourceIdx]), 1 )
ENDIF

&& Delete values from source array, adjust it's size, and reset index
=ADEL(&cSourceArray, nSourceIdx)
nSourceRows = nSourceRows - 1
nSourceIdx =  IIF(nSourceIdx - 1 = 0, 1, nSourceIdx - 1)

&& Delete row from source array
IF nSourceRows = 0
	IF nColumns > 1
		DIMENSION &cSourceArray[1, nColumns]
	ELSE
		DIMENSION &cSourceArray[1]
	ENDIF
	&cSourceArray = ""
	nSourceIdx = 1
ELSE
	IF nColumns > 1
		DIMENSION &cSourceArray[nSourceRows, nColumns]
	ELSE
		DIMENSION &cSourceArray[nSourceRows]
	ENDIF
ENDIF